package fr.faraphel.consumer.kafka

import org.apache.kafka.clients.consumer.*
import org.apache.kafka.common.serialization.DoubleDeserializer
import org.apache.kafka.common.serialization.StringDeserializer

import java.time.Duration
import java.util.Collections
import java.util.Properties
import kotlin.time.Duration.Companion.seconds
import kotlin.time.toJavaDuration


/**
 * A basic consumer that call a function on all received messages
 * @param server the kafka server address
 * @param topic the topic to get the records from
 * @param identifier the kafka identifier for the configuration
 * @param callback the function to call on all messages
 */
class Consumer(
    private val server: String,
    private val identifier: String = "fr/faraphel/consumer",
    private val topic: String,
    private val callback: (ConsumerRecord<String, Double>) -> Unit,
) {
    private val properties: Properties = Properties().apply {
        // identifier
        this[ConsumerConfig.GROUP_ID_CONFIG] = identifier
        // serializers
        this[ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG] = StringDeserializer::class.java.name
        this[ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG] = DoubleDeserializer::class.java.name
        // the server information
        this[ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG] = server
    }

    private val consumer = KafkaConsumer<String, Double>(this.properties).apply {
        // subscribe this consumer to the selected topic to receive its messages
        this.subscribe(Collections.singletonList(topic))
    }

    /**
     * Look for new messages in the selected topic and display them.
     * @param timeout the timeout for the messages to be read
     */
    fun consume(timeout: Duration = 1.seconds.toJavaDuration()) {
        // receive the new messages
        val messages: ConsumerRecords<String, Double> = this.consumer.poll(timeout)

        // print them with their timestamp and content
        messages.forEach(this.callback)
    }

    /**
     * Look indefinitely for new messages in the selected topic and display them.
     * @param timeout the timeout for the messages to be read
     */
    fun consumeForever(timeout: Duration = 1.seconds.toJavaDuration()) {
        while (true) this.consume(timeout)
    }
}
