package fr.faraphel.converter.kafka

import org.apache.kafka.common.serialization.Serdes
import org.apache.kafka.streams.KafkaStreams
import org.apache.kafka.streams.StreamsBuilder
import org.apache.kafka.streams.StreamsConfig
import org.apache.kafka.streams.Topology
import org.apache.kafka.streams.kstream.KStream
import java.util.*


/**
 * A kafka converter.
 * Take a value from a topic and convert it into another value in a second topic thanks to a conversion function.
 * @param server the kafka server address
 * @param identifier the kafka identifier for the configuration
 * @param inputTopic the input kafka topic name
 * @param outputTopic the output kafka topic name
 */
class Converter(
    private val server: String,
    private val identifier: String = "fr/faraphel/converter",
    private val inputTopic: String,
    private val outputTopic: String,
    private val converter: (Double) -> Double,
) {

    private val properties = Properties().apply {
        // set the identifier
        this[StreamsConfig.APPLICATION_ID_CONFIG] = identifier
        // set the serializers
        this[StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG] = Serdes.StringSerde::class.java.name
        this[StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG] = Serdes.DoubleSerde::class.java.name
        // set the server information
        this[StreamsConfig.BOOTSTRAP_SERVERS_CONFIG] = server
    }

    // define a topology for our streams model
    private val topologyBuilder = StreamsBuilder().apply {
        // the input stream take values from the input topic
        val inputStream: KStream<String, Double> = this.stream(inputTopic)
        // define an output stream as a stream taking values from the input stream and applying the conversion function
        val outputStream: KStream<String, Double> = inputStream.mapValues(converter)
        // set the output of the stream to the configured setup
        outputStream.to(outputTopic)
    }
    // create this topology
    private val topology: Topology = this.topologyBuilder.build()

    // create the streams system from the topology and the properties
    private val streams: KafkaStreams = KafkaStreams(this.topology, this.properties)

    /**
     * Start the conversion process
     * @see KafkaStreams.start
     */
    fun start() = this.streams.start()

    /**
     * Pause the conversion process
     * @see KafkaStreams.pause
     */
    fun pause() = this.streams.pause()

    /**
     * Resume a paused conversion process
     * @see KafkaStreams.resume
     */
    fun resume() = this.streams.resume()

    /**
     * Stop a conversion process
     * @see KafkaStreams.close
     */
    fun stop() = this.streams.close()

    /**
     * Indicate if the conversion is paused
     * @see KafkaStreams.isPaused
     */
    val isPaused: Boolean
        get() = this.streams.isPaused
}
