package fr.faraphel.producer.kafka

import com.google.gson.Gson
import com.google.gson.JsonObject
import fr.faraphel.common.error.http.HttpException
import org.apache.kafka.clients.producer.KafkaProducer
import org.apache.kafka.clients.producer.ProducerConfig
import org.apache.kafka.clients.producer.ProducerRecord
import org.apache.kafka.common.serialization.DoubleSerializer
import org.apache.kafka.common.serialization.StringSerializer
import java.util.Properties
import java.time.Duration
import kotlin.time.Duration.Companion.seconds
import kotlin.time.toJavaDuration


/**
 * A kafka temperature producer.
 * Produced values are fetched from the [Open-Meteo API](https://api.open-meteo.com)
 * @param server the kafka server address
 * @param identifier the kafka identifier for the configuration
 * @param latitude the latitude of the temperature to fetch
 * @param longitude the longitude of the temperature to fetch
 * @param topic the kafka topic to publish the data to
 */
class TemperatureProducer(
    private val server: String,
    private val identifier: String = "consumer",
    private val latitude: Double,
    private val longitude: Double,
    private val topic: String,
) {
    companion object {
        private val BASE_API_BUILDER: okhttp3.HttpUrl.Builder = okhttp3.HttpUrl.Builder()  ///< the Url builder for the API
            .scheme("https")  // use the https protocol
            .host("api.open-meteo.com")  // the api url
            .addPathSegment("v1")  // version 1
            .addPathSegment("forecast")  // forecast section of the API
            .addQueryParameter("current", "temperature")  // request the current temperature
    }

    private val jsonParser: Gson = Gson()  ///< the json parser

    private val properties: Properties = Properties().apply {  ///< the kafka producer properties
        // set the identifier
        this[ProducerConfig.CLIENT_ID_CONFIG] = identifier
        // set the serializers
        this[ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG] = StringSerializer::class.java.name
        this[ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG] = DoubleSerializer::class.java.name
        // set the server information
        this[ProducerConfig.BOOTSTRAP_SERVERS_CONFIG] = server
    }

    private val kafkaProducer: KafkaProducer<String, Double> = KafkaProducer(this.properties)  ///< the kafka producer

    private val httpClient: okhttp3.OkHttpClient = okhttp3.OkHttpClient.Builder()  ///< the HTTP client
        .callTimeout(10.seconds.toJavaDuration())
        .retryOnConnectionFailure(true)
        .build()

    private val apiUrl: okhttp3.HttpUrl = BASE_API_BUILDER  // the Weather API url
        .addQueryParameter("latitude", this.latitude.toString())
        .addQueryParameter("longitude", this.longitude.toString())
        .build()

    private val apiRequest: okhttp3.Request = okhttp3.Request.Builder()  // to API request to retrieve the data
        .get()
        .url(this.apiUrl)
        .build()


    /**
     * Produce the current temperature at the given position into a kafka server.
     * @throws HttpException the API call encountered an HTTP error.
     */
    fun produce() {
        // request the current weather data
        val response = httpClient.newCall(this.apiRequest).execute()
        // check for a successful response
        if (!response.isSuccessful)
        // in case of error, raise a http exception
            throw HttpException(response.code, response.message)

        // parse the response into a map of string to string
        val data: JsonObject = jsonParser.fromJson(response.body!!.string(), JsonObject::class.java)
        // access the current temperature
        val currentTemperature: Double = data["current"].asJsonObject["temperature"].asDouble

        // create a new record
        val record: ProducerRecord<String, Double> = ProducerRecord(this.topic, currentTemperature)
        // send the record
        this.kafkaProducer.send(record)
    }

    /**
     * indefinitely produce temperature data.
     * @param frequency the refresh rate of the data
     * @throws HttpException the API encountered an HTTP error.
     */
    fun produceForever(frequency: Duration = 60.seconds.toJavaDuration()) {
        while (true) {
            // produce a data
            this.produce()
            // wait for the cooldown
            Thread.sleep(frequency)
        }
    }
}
